/***************************************************************************
 *
 *	File		: devbar.c
 *
 *	Title		: Device Bar.
 *
 *	Description	: Provides the list of valid network devices on the system.
 *
 *	Author		: Andrae Muys
 *
 *	Revisions	: 2nd Nov 1999  AWM - Initial release.
 *
 ****************************************************************************/

#include <gnome.h>
#include <stdio.h>

#include "devbar.h"
#include "snif.h"
#include "support.h"
#include "packet.h"

#define MAXDEVLINE 512

extern GtkWidget *mainWindow;

static GtkWidget *devbar = NULL;
static GtkWidget *stop_icon;

#define poiu fprintf (stderr, "Checkpoint %s:%d\n", __FILE__, __LINE__)

static void openDevice (GtkButton *button, char *dev);
static void closeDevice (GtkButton *button, Snif *snif);

void devbarRefresh (void)
{
	GList *child;
	GtkToolbarChild *el;
	FILE *procnet;
	char devline[MAXDEVLINE];
	char *dev;
	GtkWidget *widget;
	GtkWidget *start_icon;

	if (!devbar) {
		devbar = lookup_widget (mainWindow, "deviceBar");
		stop_icon = gnome_stock_pixmap_widget 
				(mainWindow, GNOME_STOCK_PIXMAP_TIMER_STOP);
	}

	while ( (child = GTK_TOOLBAR(devbar)->children->next->next) ) {
		el = (GtkToolbarChild *)(child->data);
		GTK_TOOLBAR(devbar)->children = g_list_remove_link 
				(GTK_TOOLBAR(devbar)->children, child);
		if (el->widget) {
			gtk_object_unref (GTK_OBJECT(el->widget));
		}
		g_list_free_1 (child);
	}

	procnet = fopen ("/proc/net/dev", "r");
	if (!procnet) {
		perror ("devbar");
		return;
	}

	fgets (devline, MAXDEVLINE, procnet); /* Clear first heading line */
	fgets (devline, MAXDEVLINE, procnet); /* Clear second heading line */
	while (fgets (devline, MAXDEVLINE, procnet) != NULL) {
		strtok (devline, ":");
		dev = g_strdup (g_strchug(devline));
		start_icon = gnome_stock_pixmap_widget 
				(mainWindow, GNOME_STOCK_PIXMAP_TIMER);
		widget = gtk_toolbar_append_element (GTK_TOOLBAR(devbar),
				GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
				NULL,
				dev,
				NULL, NULL,
				start_icon, NULL,
				NULL);
		gtk_signal_connect (GTK_OBJECT(widget), "clicked",
				GTK_SIGNAL_FUNC(openDevice), dev);
	}
}

static void openDevice (GtkButton *button, char *dev)
{
	Snif *snif;

	snif = snifOpenLive (dev, ETHERNET);
	snif->tag = gdk_input_add (snif->sd, GDK_INPUT_READ, handle_packet, snif);
	gtk_signal_disconnect_by_data (GTK_OBJECT(button), dev);
	gtk_signal_connect (GTK_OBJECT(button), "clicked", 
			GTK_SIGNAL_FUNC(closeDevice), snif);
	gtk_signal_emit_stop_by_name (GTK_OBJECT(button), "clicked");
}

static void closeDevice (GtkButton *button, Snif *snif)
{
	gdk_input_remove (snif->tag);
	snifClose (snif);
	gtk_signal_disconnect_by_data (GTK_OBJECT(button), snif);
	gtk_signal_connect (GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(openDevice), snif->dev);
	free (snif);
	gtk_signal_emit_stop_by_name (GTK_OBJECT(button), "clicked");
}
