/****************************************************************************
 *
 *	File		: ipdata.c
 *
 *	Title		: IP protocol Data Engine.
 *
 *	Author		: Andrae Muys
 *
 *	Revisions	: 2nd Nov 1999  AWM - Initial release.
 *
 ****************************************************************************/

#include <glib.h>
#include <netinet/ip.h>
#include <net/ethernet.h>
#include <byteswap.h>
#include <string.h>
#include <time.h>
#include <stdio.h>

#include "ipdata.h"
#include "snif.h"
#include "show.h"

static ipData *ipTableLookup (IPTable *table, u_int32_t ip);

/****** EXTERNAL FUNCTIONS ******/

IPTable *ipTableInit ()
{
	IPTable *table;

	table = g_malloc (sizeof(IPTable));
	table->table = g_hash_table_new (NULL, NULL);
	table->chunk = g_mem_chunk_new ("IPTable", sizeof(ipData), 4096,
			G_ALLOC_ONLY);

	return table;
}

void ipTableUpdate (Snif *snif)
{
	ipData *src;
	ipData *dest;
	struct iphdr *iph = (struct iphdr *)snif->nethdr;

	src = ipTableLookup (ipTable, bswap_32 (iph->saddr));
	dest = ipTableLookup (ipTable, bswap_32 (iph->daddr));

	src->dataout += bswap_16 (iph->tot_len);
	src->lastout = time (NULL);
	dest->datain += bswap_16 (iph->tot_len);
	dest->lastin = src->lastout;

	showIPUpdate (src);
	if (src != dest) {
		showIPUpdate (dest);
	}
}

/****** STATIC FUNCTIONS ******/

static ipData *ipTableLookup (IPTable *table, u_int32_t ip)
{
	ipData *data;

	data = (ipData *)g_hash_table_lookup (table->table, GUINT_TO_POINTER(ip));

	if (!data) {
		data = g_mem_chunk_alloc (table->chunk);
		data->ip = ip;
		data->datain = 0;
		data->lastin = 0;
		data->dataout = 0;
		data->lastout = 0;
		data->row = NULL;

		g_hash_table_insert (table->table, GUINT_TO_POINTER(ip), data);
	}

	return data;
}
