/****************************************************************************
 *
 *	File		: show.c
 *
 *	Title		: User Interface Updating.
 *
 *	Description	: Note the ugly hack required to avoid repeated searches of the
 *					GtkCList for rows/cells.  This needs a revision of GtkCList
 *					to fix.
 *
 *	Author		: Andrae Muys
 *
 *	Revisions	: 2nd Nov 1999  AWM - Initial release.
 *
 ****************************************************************************/


#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <byteswap.h>
#include <time.h>

#include <gtk/gtk.h>
#include "support.h"
#include "snif.h"
#include "show.h"
#include "tcpdata.h"
#include "udpdata.h"

extern GtkWidget *mainWindow;


#define rawtoIPstr(ip) "%u.%u.%u.%u", \
	((unsigned char *)&(ip))[3], \
	((unsigned char *)&(ip))[2], \
	((unsigned char *)&(ip))[1], \
	((unsigned char *)&(ip))[0]

void showConnUpdate (connData *conn)
{
	static char *row[8];
	static char rows[8][30];
	static GtkWidget *table = NULL;
	int i;

	if (!table) {
		table = lookup_widget (mainWindow, "tcpList");
		/*
		 * Used to avoid the extra mallocs required if I use strdup.
		 */
		row[0] = &(rows[0][0]);
		row[1] = &(rows[1][0]);
		row[2] = &(rows[2][0]);
		row[3] = &(rows[3][0]);
		row[4] = &(rows[4][0]);
		row[5] = &(rows[5][0]);
		row[6] = &(rows[6][0]);
		row[7] = &(rows[7][0]);
	}

	sprintf (row[0], rawtoIPstr (conn->hdr->ip1));
	sprintf (row[1], "%u", conn->hdr->port1);
	sprintf (row[2], rawtoIPstr (conn->hdr->ip2));
	sprintf (row[3], "%u", conn->hdr->port2);
	sprintf (row[4], "%lu", conn->volume);
	sprintf (row[5], "%s", ctime (&conn->start));
	sprintf (row[6], "%s", ctime (&conn->last));
	sprintf (row[7], "%s", tcpStateStr[conn->state]);

	if (conn->row) {
		gtk_clist_freeze (GTK_CLIST(table));
		for (i = 0; i < 8; i++) {
			g_free (conn->row->cell[i].u.text);
			conn->row->cell[i].u.text = g_strdup (row[i]);
		}
		gtk_clist_thaw (GTK_CLIST(table));
	} else {
		gtk_clist_prepend (GTK_CLIST(table), row);
		conn->row = GTK_CLIST(table)->row_list->data;
	}
}

void showUDPUpdate (UDPData *udp)
{
	static char *row[5];
	static char rows[5][30];
	static GtkWidget *table = NULL;
	int i;

	if (!table) {
		table = lookup_widget (mainWindow, "udpList");
		/*
		 * Used to avoid the extra mallocs required if I use strdup.
		 */
		row[0] = &(rows[0][0]);
		row[1] = &(rows[1][0]);
		row[2] = &(rows[2][0]);
		row[3] = &(rows[3][0]);
		row[4] = &(rows[4][0]);
	}

	sprintf (row[0], rawtoIPstr (udp->ip));
	sprintf (row[1], "%lu", udp->datain);
	sprintf (row[2], "%lu", udp->dataout);
	sprintf (row[3], "%s", ctime (&udp->lastin));
	sprintf (row[4], "%s", ctime (&udp->lastout));

	/*
	 * Ok I know this is really nasty, but without it we have to use the builtin
	 * search functions which are really crappy.  For now we save a pointer to
	 * the row in the table, modify the rows text fields directly, and use 
	 * the _freeze/_thaw functions to trigger an update.
	 */
	if (udp->row) {
		gtk_clist_freeze (GTK_CLIST(table));
		for (i = 0; i < 5; i++) {
			g_free (udp->row->cell[i].u.text);
			udp->row->cell[i].u.text = g_strdup (row[i]);
		}
		gtk_clist_thaw (GTK_CLIST(table));
	} else {
		gtk_clist_prepend (GTK_CLIST(table), row);
		udp->row = GTK_CLIST(table)->row_list->data;
	}
}

void showIPUpdate (ipData *ip)
{
	static char *row[5];
	static char rows[5][30];
	static GtkWidget *table = NULL;
	int i;

	if (!table) {
		table = lookup_widget (mainWindow, "ipList");
		/*
		 * Used to avoid the extra mallocs required if I use strdup.
		 */
		row[0] = &(rows[0][0]);
		row[1] = &(rows[1][0]);
		row[2] = &(rows[2][0]);
		row[3] = &(rows[3][0]);
		row[4] = &(rows[4][0]);
	}

	sprintf (row[0], rawtoIPstr (ip->ip));
	sprintf (row[1], "%lu", ip->datain);
	sprintf (row[2], "%s", ctime (&ip->lastin));
	sprintf (row[3], "%lu", ip->dataout);
	sprintf (row[4], "%s", ctime (&ip->lastout));

	if (ip->row) {
		gtk_clist_freeze (GTK_CLIST(table));
		for (i = 0; i < 5; i++) {
			g_free (ip->row->cell[i].u.text);
			ip->row->cell[i].u.text = g_strdup (row[i]);
		}
		gtk_clist_thaw (GTK_CLIST(table));
	} else {
		gtk_clist_prepend (GTK_CLIST(table), row);
		ip->row = GTK_CLIST(table)->row_list->data;
	}
}

/*
 * Currently used in arpdata.h, probably should be here, but unsure how to do
 * that nicely without having to do multiple O(n) searches of the tree.
 *
 * #define rawtoETHstr(addr) "%02x:%02x:%02x:%02x:%02x:%02x", \
 *	((char *)&(addr))[0], ((char *)&(addr))[1], ((char *)&(addr))[2], \
 *	((char *)&(addr))[3], ((char *)&(addr))[4], ((char *)&(addr))[5] 
 */

void showARPUpdate (arpData *arp)
{
	static GtkWidget *arpTree = NULL;
/*	char iplabel[16]; */
	GList *curr;

	if (!arpTree) {
		arpTree = lookup_widget (mainWindow, "arpTree");
	}

#if 0	/* This code commented out as it is now handled by arpdata. */
	if (!arp->node) {
		g_snprintf (iplabel, 16, rawtoETHstr (arp->update->data));
		arp->node = gtk_tree_item_new_with_label (iplabel);
	}
#endif
	gtk_tree_remove_items (GTK_TREE(arpTree), arp->update);
	g_list_reverse (arp->update);
	while ( (curr = arp->update) ) {
		gtk_tree_prepend (GTK_TREE(arpTree), GTK_WIDGET(curr->data));
		arp->update = g_list_remove_link (arp->update, curr);
		g_list_free (curr);
	}
}
