/****************************************************************************
 *
 *	File		: snif.c
 *
 *	Title		: Packet Capture Engine.
 *
 *	Description	: This currently uses the old SOCK_PACKET interface.  This has
 *					to change.  Too bad I couldn't get the new PF_PACKET
 *					interface to work in the time I had :(.  Oh well I'll fix it
 *					after I get the ARP and DNS working.
 *
 *	Author		: Andrae Muys
 *
 *	Revisions	: 2nd Nov 1999  AWM - Initial release.
 *
 ****************************************************************************/

#include <sys/socket.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <net/if.h>
#include <net/ethernet.h>
#include <stdio.h>

#include "snif.h"

Snif *snifOpenLive (char *dev, int proto)
{
	Snif *snif;
	struct ifreq ifr;
	struct sockaddr sa;

	snif = (Snif *)malloc (sizeof(Snif));
	if (!snif) goto error;

	snif->dev = dev;
	snif->llproto = proto;
	/*
	 * There are ioctl's to find all this stuff, but they can wait till the
	 * rest of the code works.
	 */
	switch (proto) {
	case ETHERNET :
		snif->mtu = 1500;
		snif->buf = (char *)malloc (snif->mtu);
		if (!snif->buf) goto error;
		break;
	default :
		fprintf (stderr, "Unknown protocol\n");
		return NULL;
	}

	/* 
	 * This is the old, depreciated interface.  This should be replaced by
	 * a PF_PACKET socket once the rest of the program works.
	 */
	snif->sd = socket (PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
	if (!snif->sd) goto error;

#if 0
	/* See socket.4 */
	setsockopt (snif->sd, SOL_SOCKET, SO_BINDTODEVICE, dev, strlen(dev));
#endif
	/* 
	 * This should be replaced by setsockopt (see above), but for some reason
	 * setsockopt isn't working so I am continuting to use the old bind()
	 * interface in the meantime.
	 */
    memset (&sa, 0, sizeof(sa));
	sa.sa_family = AF_INET;
	strncpy (sa.sa_data, dev, sizeof(sa.sa_data));
	bind (snif->sd, &sa, sizeof(sa));

	/* See netdevice.4 */
	memset (&ifr, 0, sizeof(ifr));
	strcpy (ifr.ifr_name, dev);
	ioctl (snif->sd, SIOCGIFFLAGS, &ifr);
	snif->ifr = ifr;

	ifr.ifr_flags |= IFF_PROMISC;
	ioctl (snif->sd, SIOCSIFFLAGS, &ifr);
	on_exit (restore_ifr, snif);

	return snif;
error:
	perror ("sniff");
	return NULL;
}

void snifClose (Snif *snif)
{
	restore_ifr (0, snif);
	close (snif->sd);
}

/*
 * This needs to be attached to every signal that can cause pasmon to exit.
 */
void restore_ifr (int val, void *data)
{
	Snif *snif = (Snif *)data;
	
	ioctl (snif->sd, SIOCSIFFLAGS, &(snif->ifr));
}
