/****************************************************************************
 *
 *	File		: udpdata.c
 *
 *	Title		: UDP Protocol Data Engine.
 *
 *	Author		: Andrae Muys
 *
 *	Revisions	: 2nd Nov 1999  AWM - Initial release.
 *
 ****************************************************************************/

#include <glib.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <net/ethernet.h>
#include <byteswap.h>
#include <string.h>
#include <time.h>
#include <stdio.h>

#include "udpdata.h"
#include "snif.h"
#include "show.h"

static UDPData *udpTableLookup (UDPTable *table, u_int32_t ip);

/****** EXTERNAL FUNCTIONS ******/

UDPTable *udpTableInit ()
{
	UDPTable *table;

	table = g_malloc (sizeof(UDPTable));
	table->table = g_hash_table_new (NULL, NULL);
	table->chunk = g_mem_chunk_new ("UDPTable", sizeof(UDPData), 4096,
			G_ALLOC_ONLY);

	return table;
}

void udpTableUpdate (Snif *snif)
{
	UDPData *src, *dest;
	struct iphdr *iph = (struct iphdr *)snif->nethdr;
	struct udphdr *udph = (struct udphdr *)snif->tpthdr;

	src = udpTableLookup (udpTable, bswap_32 (iph->saddr));
	dest = udpTableLookup (udpTable, bswap_32 (iph->daddr));

	src->dataout += bswap_16 (udph->len);
	src->lastout = time (NULL);
	dest->datain += bswap_16 (udph->len);
	dest->lastin = src->lastout;

	showUDPUpdate (src);
	showUDPUpdate (dest);
}

/****** STATIC FUNCTIONS ******/

static UDPData *udpTableLookup (UDPTable *table, u_int32_t ip)
{
	UDPData *data;

	data = (UDPData *)g_hash_table_lookup (table->table, GUINT_TO_POINTER(ip));

	if (!data) {
		data = g_mem_chunk_alloc (table->chunk);
		data->ip = ip;
		data->datain = 0;
		data->dataout = 0;
		data->row = NULL;

		g_hash_table_insert (table->table, GUINT_TO_POINTER(ip), data);
	}

	return data;
}
